@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -52,13 +52,13 @@ extern "C"
 {
 #endif
 
-#define GNUTLS_VERSION "3.1.7"
+#define GNUTLS_VERSION "3.2.1"
 
 #define GNUTLS_VERSION_MAJOR 3
-#define GNUTLS_VERSION_MINOR 1
-#define GNUTLS_VERSION_PATCH 7
+#define GNUTLS_VERSION_MINOR 2
+#define GNUTLS_VERSION_PATCH 1
 
-#define GNUTLS_VERSION_NUMBER 0x030107
+#define GNUTLS_VERSION_NUMBER 0x030201
 
 #define GNUTLS_CIPHER_RIJNDAEL_128_CBC GNUTLS_CIPHER_AES_128_CBC
 #define GNUTLS_CIPHER_RIJNDAEL_256_CBC GNUTLS_CIPHER_AES_256_CBC
@@ -111,6 +111,8 @@ extern "C"
     GNUTLS_CIPHER_AES_128_GCM = 93,
     GNUTLS_CIPHER_AES_256_GCM = 94,
     GNUTLS_CIPHER_CAMELLIA_192_CBC = 95,
+    GNUTLS_CIPHER_SALSA20_256 = 96,
+    GNUTLS_CIPHER_ESTREAM_SALSA20_256 = 97,
 
     /* used only for PGP internals. Ignored in TLS/SSL
      */
@@ -136,7 +138,7 @@ extern "C"
  * @GNUTLS_KX_ANON_DH: Anon-DH key-exchange algorithm.
  * @GNUTLS_KX_ANON_ECDH: Anon-ECDH key-exchange algorithm.
  * @GNUTLS_KX_SRP: SRP key-exchange algorithm.
- * @GNUTLS_KX_RSA_EXPORT: RSA-EXPORT key-exchange algorithm.
+ * @GNUTLS_KX_RSA_EXPORT: RSA-EXPORT key-exchange algorithm (defunc).
  * @GNUTLS_KX_SRP_RSA: SRP-RSA key-exchange algorithm.
  * @GNUTLS_KX_SRP_DSS: SRP-DSS key-exchange algorithm.
  * @GNUTLS_KX_PSK: PSK key-exchange algorithm.
@@ -166,7 +168,7 @@ extern "C"
 
 /**
  * gnutls_params_type_t:
- * @GNUTLS_PARAMS_RSA_EXPORT: Session RSA-EXPORT parameters.
+ * @GNUTLS_PARAMS_RSA_EXPORT: Session RSA-EXPORT parameters (defunc).
  * @GNUTLS_PARAMS_DH: Session Diffie-Hellman parameters.
  * @GNUTLS_PARAMS_ECDH: Session Elliptic-Curve Diffie-Hellman parameters.
  *
@@ -232,7 +234,9 @@ extern "C"
     GNUTLS_MAC_SHA224 = 9,
       /* If you add anything here, make sure you align with
          gnutls_digest_algorithm_t. */
-    GNUTLS_MAC_AEAD = 200 /* indicates that MAC is on the cipher */
+    GNUTLS_MAC_AEAD = 200, /* indicates that MAC is on the cipher */
+    GNUTLS_MAC_UMAC_96 = 201,
+    GNUTLS_MAC_UMAC_128 = 202,
   } gnutls_mac_algorithm_t;
 
 /**
@@ -384,6 +388,7 @@ extern "C"
     GNUTLS_A_CERTIFICATE_UNOBTAINABLE = 111,
     GNUTLS_A_UNRECOGNIZED_NAME = 112,
     GNUTLS_A_UNKNOWN_PSK_IDENTITY = 115,
+    GNUTLS_A_NO_APPLICATION_PROTOCOL = 120,
   } gnutls_alert_description_t;
 
 /**
@@ -522,6 +527,7 @@ extern "C"
  * @GNUTLS_TLS1_1: TLS version 1.1.
  * @GNUTLS_TLS1_2: TLS version 1.2.
  * @GNUTLS_DTLS1_0: DTLS version 1.0.
+ * @GNUTLS_DTLS1_2: DTLS version 1.2.
  * @GNUTLS_DTLS0_9: DTLS version 0.9 (Cisco AnyConnect / OpenSSL 0.9.8e).
  * @GNUTLS_VERSION_MAX: Maps to the highest supported TLS version.
  * @GNUTLS_VERSION_UNKNOWN: Unknown SSL/TLS version.
@@ -535,10 +541,13 @@ extern "C"
     GNUTLS_TLS1 = GNUTLS_TLS1_0,
     GNUTLS_TLS1_1 = 3,
     GNUTLS_TLS1_2 = 4,
-    GNUTLS_DTLS1_0 = 5,
-    GNUTLS_DTLS0_9 = 6,
-    GNUTLS_VERSION_MAX = GNUTLS_DTLS0_9,
-    GNUTLS_VERSION_UNKNOWN = 0xff
+    GNUTLS_DTLS0_9 = 6, /* FIXME: at some point change it to 200 */
+    GNUTLS_DTLS1_0 = 5, /* 201 */
+    GNUTLS_DTLS1_2 = 202,
+    GNUTLS_DTLS_VERSION_MIN = GNUTLS_DTLS1_0,
+    GNUTLS_DTLS_VERSION_MAX = GNUTLS_DTLS1_2,
+    GNUTLS_TLS_VERSION_MAX = GNUTLS_TLS1_2,
+    GNUTLS_VERSION_UNKNOWN = 0xff /* change it to 0xffff */
   } gnutls_protocol_t;
 
 /**
@@ -809,14 +818,12 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
     gnutls_certificate_type_get (gnutls_session_t session);
     
   int gnutls_sign_algorithm_get (gnutls_session_t session);
+  int gnutls_sign_algorithm_get_client (gnutls_session_t session);
 
   int gnutls_sign_algorithm_get_requested (gnutls_session_t session,
                                            size_t indx,
                                            gnutls_sign_algorithm_t * algo);
 
-  size_t gnutls_cipher_get_key_size (gnutls_cipher_algorithm_t algorithm);
-  size_t gnutls_mac_get_key_size (gnutls_mac_algorithm_t algorithm);
-
 /* the name of the specified algorithms */
   const char *gnutls_cipher_get_name (gnutls_cipher_algorithm_t algorithm);
   const char *gnutls_mac_get_name (gnutls_mac_algorithm_t algorithm);
@@ -827,6 +834,9 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
                                                 type);
   const char *gnutls_pk_get_name (gnutls_pk_algorithm_t algorithm);
   const char *gnutls_sign_get_name (gnutls_sign_algorithm_t algorithm);
+
+  size_t gnutls_cipher_get_key_size (gnutls_cipher_algorithm_t algorithm);
+  size_t gnutls_mac_get_key_size (gnutls_mac_algorithm_t algorithm);
   
   int gnutls_sign_is_secure (gnutls_sign_algorithm_t algorithm);
   gnutls_digest_algorithm_t
@@ -877,6 +887,8 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
  */
   void gnutls_handshake_set_private_extensions (gnutls_session_t session,
                                                 int allow);
+  int gnutls_handshake_set_random (gnutls_session_t session, const gnutls_datum_t* random);
+
     gnutls_handshake_description_t
     gnutls_handshake_get_last_out (gnutls_session_t session);
     gnutls_handshake_description_t
@@ -1038,6 +1050,14 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
   int gnutls_srtp_set_mki (gnutls_session_t session, const gnutls_datum_t *mki);
   int gnutls_srtp_get_mki (gnutls_session_t session, gnutls_datum_t *mki);
 
+/* ALPN TLS extension */
+#define GNUTLS_ALPN_MAND 1
+  int gnutls_alpn_get_selected_protocol (gnutls_session_t session,
+                                   gnutls_datum_t * protocol);
+  int gnutls_alpn_set_protocols (gnutls_session_t session,
+                           const gnutls_datum_t * protocols, unsigned protocols_size,
+                           unsigned flags);
+
   int gnutls_key_generate (gnutls_datum_t * key, unsigned int key_size);
 
 /* if you just want some defaults, use the following.
@@ -1088,6 +1108,7 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
                                 gnutls_datum_t * data);
   void gnutls_session_get_random (gnutls_session_t session, gnutls_datum_t* client, 
                                   gnutls_datum_t* server);
+  char * gnutls_session_get_desc (gnutls_session_t session);
 
   int gnutls_session_set_premaster (gnutls_session_t session, unsigned int entity,
                            gnutls_protocol_t version, 
@@ -1104,6 +1125,9 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
                              size_t * session_id_size);
   int gnutls_session_get_id2 (gnutls_session_t session, gnutls_datum_t *session_id);
 
+  int gnutls_session_set_id (gnutls_session_t session, 
+                              const gnutls_datum_t * sid);
+
   int gnutls_session_channel_binding (gnutls_session_t session,
                                       gnutls_channel_binding_t cbtype,
                                       gnutls_datum_t * cb);
@@ -1158,6 +1182,12 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
 
 /* x.509 types */
 
+  struct gnutls_pubkey_st;
+  typedef struct gnutls_pubkey_st *gnutls_pubkey_t;
+
+  struct gnutls_privkey_st;
+  typedef struct gnutls_privkey_st *gnutls_privkey_t;
+
   struct gnutls_x509_privkey_int;
   typedef struct gnutls_x509_privkey_int *gnutls_x509_privkey_t;
 
@@ -1260,11 +1290,26 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
                                           const char *keyfile,
                                           gnutls_x509_crt_fmt_t type);
 
+  int
+    gnutls_certificate_set_x509_key_file2 (gnutls_certificate_credentials_t
+                                          res, const char *certfile,
+                                          const char *keyfile,
+                                          gnutls_x509_crt_fmt_t type,
+                                          const char* pass,
+                                          unsigned int flags);
+
   int gnutls_certificate_set_x509_key_mem (gnutls_certificate_credentials_t
                                            res, const gnutls_datum_t * cert,
                                            const gnutls_datum_t * key,
                                            gnutls_x509_crt_fmt_t type);
 
+  int gnutls_certificate_set_x509_key_mem2 (gnutls_certificate_credentials_t
+                                           res, const gnutls_datum_t * cert,
+                                           const gnutls_datum_t * key,
+                                           gnutls_x509_crt_fmt_t type,
+                                           const char* pass,
+                                           unsigned int flags);
+
   void gnutls_certificate_send_x509_rdn_sequence (gnutls_session_t session,
                                                   int status);
 
@@ -1408,6 +1453,15 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session);
 
   typedef int (*gnutls_errno_func) (gnutls_transport_ptr_t);
 
+/* This will be defined as macro.
+  void gnutls_transport_set_int (gnutls_session_t session, int r);
+ */
+  void gnutls_transport_set_int2 (gnutls_session_t session, int r, int s);
+  #define gnutls_transport_set_int(s, i) gnutls_transport_set_int2(s, i, i)
+  
+  void gnutls_transport_get_int2 (gnutls_session_t session, int * r, int *s);
+  int gnutls_transport_get_int (gnutls_session_t session);
+
   void gnutls_transport_set_ptr (gnutls_session_t session,
                                  gnutls_transport_ptr_t ptr);
   void gnutls_transport_set_ptr2 (gnutls_session_t session,
@@ -2153,6 +2207,7 @@ typedef int (*gnutls_pin_callback_t) (void *userdata, int attempt,
 #define GNUTLS_E_OCSP_RESPONSE_ERROR -341
 #define GNUTLS_E_RANDOM_DEVICE_ERROR -342
 #define GNUTLS_E_AUTH_ERROR -343
+#define GNUTLS_E_NO_APPLICATION_PROTOCOL -344
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -1250
 
