@@ -9,11 +9,11 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
+#include <gnutls/abstract.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -26,50 +26,20 @@
 
 #define CERT_FILE "cert.pem"
 #define KEY_FILE "key.pem"
-#define CAFILE "ca.pem"
+#define CAFILE "/etc/ssl/certs/ca-certificates.crt"
 
 extern int tcp_connect (void);
 extern void tcp_close (int sd);
 
-static int cert_callback (gnutls_session_t session,
-                          const gnutls_datum_t * req_ca_rdn, int nreqs,
-                          const gnutls_pk_algorithm_t * sign_algos,
-                          int sign_algos_length, gnutls_retr2_st * st);
+static int
+cert_callback (gnutls_session_t session,
+               const gnutls_datum_t * req_ca_rdn, int nreqs,
+               const gnutls_pk_algorithm_t * sign_algos,
+               int sign_algos_length, gnutls_pcert_st ** pcert,
+               unsigned int *pcert_length, gnutls_privkey_t * pkey);
 
-gnutls_x509_crt_t crt;
-gnutls_x509_privkey_t key;
-
-/* Helper functions to load a certificate and key
- * files into memory.
- */
-static gnutls_datum_t
-load_file (const char *file)
-{
-  FILE *f;
-  gnutls_datum_t loaded_file = { NULL, 0 };
-  long filelen;
-  void *ptr;
-
-  if (!(f = fopen (file, "r"))
-      || fseek (f, 0, SEEK_END) != 0
-      || (filelen = ftell (f)) < 0
-      || fseek (f, 0, SEEK_SET) != 0
-      || !(ptr = malloc ((size_t) filelen))
-      || fread (ptr, 1, (size_t) filelen, f) < (size_t) filelen)
-    {
-      return loaded_file;
-    }
-
-  loaded_file.data = ptr;
-  loaded_file.size = (unsigned int) filelen;
-  return loaded_file;
-}
-
-static void
-unload_file (gnutls_datum_t data)
-{
-  free (data.data);
-}
+gnutls_pcert_st pcrt;
+gnutls_privkey_t key;
 
 /* Load the certificate and the private key.
  */
@@ -79,43 +49,41 @@ load_keys (void)
   int ret;
   gnutls_datum_t data;
 
-  data = load_file (CERT_FILE);
-  if (data.data == NULL)
-    {
-      fprintf (stderr, "*** Error loading cert file.\n");
-      exit (1);
-    }
-  gnutls_x509_crt_init (&crt);
-
-  ret = gnutls_x509_crt_import (crt, &data, GNUTLS_X509_FMT_PEM);
+  ret = gnutls_load_file (CERT_FILE, &data);
   if (ret < 0)
     {
-      fprintf (stderr, "*** Error loading key file: %s\n",
+      fprintf (stderr, "*** Error loading certificate file.\n");
+      exit (1);
+    }
+
+  ret = gnutls_pcert_import_x509_raw (&pcrt, &data, GNUTLS_X509_FMT_PEM, 0);
+  if (ret < 0)
+    {
+      fprintf (stderr, "*** Error loading certificate file: %s\n",
                gnutls_strerror (ret));
       exit (1);
     }
 
-  unload_file (data);
+  gnutls_free(data.data);
 
-  data = load_file (KEY_FILE);
-  if (data.data == NULL)
+  ret = gnutls_load_file (KEY_FILE, &data);
+  if (ret < 0)
     {
       fprintf (stderr, "*** Error loading key file.\n");
       exit (1);
     }
 
-  gnutls_x509_privkey_init (&key);
+  gnutls_privkey_init (&key);
 
-  ret = gnutls_x509_privkey_import (key, &data, GNUTLS_X509_FMT_PEM);
+  ret = gnutls_privkey_import_x509_raw (key, &data, GNUTLS_X509_FMT_PEM, NULL, 0);
   if (ret < 0)
     {
       fprintf (stderr, "*** Error loading key file: %s\n",
                gnutls_strerror (ret));
       exit (1);
     }
-
-  unload_file (data);
-
+    
+  gnutls_free(data.data);
 }
 
 int
@@ -144,7 +112,7 @@ main (void)
    */
   gnutls_certificate_set_x509_trust_file (xcred, CAFILE, GNUTLS_X509_FMT_PEM);
 
-  gnutls_certificate_set_retrieve_function (xcred, cert_callback);
+  gnutls_certificate_set_retrieve_function2 (xcred, cert_callback);
 
   /* Initialize TLS session 
    */
@@ -161,7 +129,7 @@ main (void)
    */
   sd = tcp_connect ();
 
-  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) sd);
+  gnutls_transport_set_int (session, sd);
 
   /* Perform the TLS handshake
    */
@@ -175,7 +143,11 @@ main (void)
     }
   else
     {
-      printf ("- Handshake was completed\n");
+      char* desc;
+      
+      desc = gnutls_session_get_desc(session);
+      printf ("- Session info: %s\n", desc);
+      gnutls_free(desc);
     }
 
   gnutls_record_send (session, MSG, strlen (MSG));
@@ -226,7 +198,8 @@ static int
 cert_callback (gnutls_session_t session,
                const gnutls_datum_t * req_ca_rdn, int nreqs,
                const gnutls_pk_algorithm_t * sign_algos,
-               int sign_algos_length, gnutls_retr2_st * st)
+               int sign_algos_length, gnutls_pcert_st ** pcert,
+               unsigned int *pcert_length, gnutls_privkey_t * pkey)
 {
   char issuer_dn[256];
   int i, ret;
@@ -256,59 +229,12 @@ cert_callback (gnutls_session_t session,
    * The certificate must be of any of the "sign algorithms"
    * supported by the server.
    */
-
   type = gnutls_certificate_type_get (session);
   if (type == GNUTLS_CRT_X509)
     {
-      /* check if the certificate we are sending is signed
-       * with an algorithm that the server accepts */
-      gnutls_sign_algorithm_t cert_algo, req_algo;
-      int i, match = 0;
-
-      ret = gnutls_x509_crt_get_signature_algorithm (crt);
-      if (ret < 0)
-        {
-          /* error reading signature algorithm 
-           */
-          return -1;
-        }
-      cert_algo = ret;
-
-      i = 0;
-      do
-        {
-          ret = gnutls_sign_algorithm_get_requested (session, i, &req_algo);
-          if (ret >= 0 && cert_algo == req_algo)
-            {
-              match = 1;
-              break;
-            }
-
-          /* server has not requested anything specific */
-          if (i == 0 && ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
-            {
-              match = 1;
-              break;
-            }
-          i++;
-        }
-      while (ret >= 0);
-
-      if (match == 0)
-        {
-          printf
-            ("- Could not find a suitable certificate to send to server\n");
-          return -1;
-        }
-
-      st->cert_type = type;
-      st->ncerts = 1;
-
-      st->cert.x509 = &crt;
-      st->key.x509 = key;
-      st->key_type = GNUTLS_PRIVKEY_X509;
-
-      st->deinit_all = 0;
+      *pcert_length = 1;
+      *pcert = &pcrt;
+      *pkey = key;
     }
   else
     {
